package com.hjy.modlue.gather;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.gather.ICbGatherService;
import com.hjy.module.vo.gather.CbGatherVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 数据采集Controller
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Api(tags = "数据采集Controller")
@RestController
@RequestMapping("/gather")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherController extends BaseController {

    private final ICbGatherService cbGatherService;

    @ApiOperation("查询数据采集列表")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:list')")
    @GetMapping("/list")
    public TableDataInfo<CbGatherVo> list(CbGatherVo entity) {
        return cbGatherService.queryList(entity);
    }

    @ApiOperation("查询数据采集所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbGatherVo entity) {
        return AjaxResult.success("查询成功", cbGatherService.queryAll(entity));
    }

    @ApiOperation("导出数据采集列表")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:export')")
    @Log(title = "数据采集", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbGatherVo entity) {
        List<CbGatherVo> list = cbGatherService.queryAll(entity);
        ExcelUtil<CbGatherVo> util = new ExcelUtil<>(CbGatherVo.class);
        util.exportExcel(response, list, "数据采集数据");
    }

    @ApiOperation("获取数据采集详细信息")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbGatherService.queryById(id));
    }

    @ApiOperation("新增数据采集")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:add')")
    @Log(title = "数据采集", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbGatherVo entity) {
        return toAjax(cbGatherService.save(entity));
    }

    @ApiOperation("修改数据采集")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:edit')")
    @Log(title = "数据采集", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbGatherVo entity) {
        return toAjax(cbGatherService.updateById(entity));
    }

    @ApiOperation("删除数据采集")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:remove')")
    @Log(title = "数据采集", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbGatherService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
