package com.hjy.modlue.gather;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.gather.ICbGatherFeeService;
import com.hjy.module.vo.gather.CbGatherFeeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 数据采集明细Controller
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Api(tags = "数据采集明细Controller")
@RestController
@RequestMapping("/gather_fee")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherFeeController extends BaseController {

    private final ICbGatherFeeService cbGatherFeeService;

    @ApiOperation("查询数据采集明细列表")
    @PreAuthorize("@ss.hasPermi('gather:GATHERFEE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbGatherFeeVo> list(CbGatherFeeVo entity) {
        return cbGatherFeeService.queryList(entity);
    }

    @ApiOperation("查询数据采集明细所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbGatherFeeVo entity) {
        return AjaxResult.success("查询成功", cbGatherFeeService.queryAll(entity));
    }

    @ApiOperation("导出数据采集明细列表")
    @PreAuthorize("@ss.hasPermi('gather:GATHERFEE:export')")
    @Log(title = "数据采集明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbGatherFeeVo entity) {
        List<CbGatherFeeVo> list = cbGatherFeeService.queryAll(entity);
        ExcelUtil<CbGatherFeeVo> util = new ExcelUtil<>(CbGatherFeeVo.class);
        util.exportExcel(response, list, "数据采集明细数据");
    }

    @ApiOperation("获取数据采集明细详细信息")
    @PreAuthorize("@ss.hasPermi('gather:GATHERFEE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbGatherFeeService.queryById(id));
    }

    @ApiOperation("新增数据采集明细")
    @PreAuthorize("@ss.hasPermi('gather:GATHERFEE:add')")
    @Log(title = "数据采集明细", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbGatherFeeVo entity) {
        return toAjax(cbGatherFeeService.save(entity));
    }

    @ApiOperation("修改数据采集明细")
    @PreAuthorize("@ss.hasPermi('gather:GATHERFEE:edit')")
    @Log(title = "数据采集明细", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbGatherFeeVo entity) {
        return toAjax(cbGatherFeeService.updateById(entity));
    }

    @ApiOperation("删除数据采集明细")
    @PreAuthorize("@ss.hasPermi('gather:GATHERFEE:remove')")
    @Log(title = "数据采集明细", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbGatherFeeService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
