package com.hjy.modlue.norm;

import cn.hutool.json.JSONUtil;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.HisModule;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.commom.HisMethods;
import com.hjy.module.service.norm.ICbNormFeeService;
import com.hjy.module.vo.norm.CbNormFeeVo;
import com.hjy.module.vo.norm.CbNormVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 指标权重明细Controller
 *
 * @author Tellsea
 * @date 2023-08-21
 */
@Api(tags = "指标权重明细Controller")
@RestController
@RequestMapping("/normFee")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbNormFeeController extends BaseController {

    private final ICbNormFeeService cbNormFeeService;

    @ApiOperation("查询指标权重明细列表")
    @PreAuthorize("@ss.hasPermi('business:NORMFEE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbNormFeeVo> list(CbNormFeeVo entity) {
        return cbNormFeeService.queryList(entity);
    }

    @ApiOperation("查询指标权重明细所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbNormFeeVo entity) {
        return AjaxResult.success("查询成功", cbNormFeeService.queryAll(entity));
    }

    @ApiOperation("导出指标权重明细列表")
    @PreAuthorize("@ss.hasPermi('business:NORMFEE:export')")
    @Log(title = "指标权重明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbNormFeeVo entity) {
        List<CbNormFeeVo> list = cbNormFeeService.queryAll(entity);
        ExcelUtil<CbNormFeeVo> util = new ExcelUtil<>(CbNormFeeVo.class);
        util.exportExcel(response, list, "指标权重明细数据");
    }

    @ApiOperation("获取指标权重明细详细信息")
    @PreAuthorize("@ss.hasPermi('business:NORMFEE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbNormFeeService.queryById(id));
    }

    @ApiOperation("新增指标权重明细")
    @PreAuthorize("@ss.hasPermi('business:NORMFEE:add')")
    @Log(title = "指标权重明细", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbNormFeeVo entity) {
        return toAjax(cbNormFeeService.save(entity));
    }

    @ApiOperation("修改指标权重明细")
    @PreAuthorize("@ss.hasPermi('business:NORMFEE:edit')")
    @Log(title = "指标权重明细", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbNormFeeVo entity) {
        boolean flag = cbNormFeeService.updateById(entity);
        if (flag){
            HisMethods.saveFromHis(HisModule.MODULE2.getCode(),entity.getId(), JSONUtil.toJsonStr(entity));
        }
        return toAjax(flag);
    }

    @ApiOperation("删除指标权重明细")
    @PreAuthorize("@ss.hasPermi('business:NORMFEE:remove')")
    @Log(title = "指标权重明细", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbNormFeeService.removeByIds(Arrays.asList(ids)));
    }
}
