package com.hjy.modlue.portalAPI;

import cn.hutool.core.util.ObjectUtil;
import com.hjy.business.appuser.utils.JsonUtils;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.constant.Constants;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.CbCompanyUser;
import com.hjy.common.core.domain.model.LoginBody;
import com.hjy.common.core.domain.model.RegisterBody;
import com.hjy.common.core.redis.RedisCache;
import com.hjy.common.exception.ServiceException;
import com.hjy.common.utils.DateUtils;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.ServletUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.ip.IpUtils;
import com.hjy.common.utils.uuid.IdUtils;
import com.hjy.framework.web.service.SysLoginService;
import com.hjy.framework.web.service.TokenService;
import com.hjy.module.service.portal.ICbCompanyUserService;
import com.hjy.web.controller.common.CaptchaUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Api(tags = "门户登录相关接口")
@RestController
@RequestMapping("/portal_api/login")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class PortalLoginController {

    @Autowired
    private CaptchaUtils captchaUtils;

    @Autowired
    private SysLoginService sysLoginService;

    @Autowired
    private ICbCompanyUserService cbCompanyUserService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private RedisCache redisCache;



    @ApiOperation("获取门户验证码")
    @GetMapping("/captchaImage")
    public AjaxResult getCode(HttpServletResponse response) throws IOException {
        return captchaUtils.getCaptchaData();
    }


    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @ApiOperation("门户登录")
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody) {
        sysLoginService.validateCaptcha(loginBody.getUsername(),loginBody.getCode(),loginBody.getUuid());
        CbCompanyUser user = cbCompanyUserService.selectCbCompanyUserByuserName(loginBody.getUsername());
        if (ObjectUtil.isNull(user)) {
            throw new ServiceException("登录用户：" + loginBody.getUsername() + " 不存在");
        }
        if (!SecurityUtils.matchesPassword(loginBody.getPassword(),user.getPassword())){
            throw new ServiceException("用户名密码错误!");
        }
        Map<String, Object> claims = new HashMap<>();
        String uuid = IdUtils.fastUUID();
        claims.put(Constants.PORTAL_USER_KEY, uuid);
        String token = tokenService.createTokenStr(claims);
        redisCache.setCacheObject(Constants.PORTAL_USER_KEY+uuid, JsonUtils.toJson(user),120, TimeUnit.MINUTES);
        AjaxResult ajax = AjaxResult.success();
        ajax.put(Constants.TOKEN, token);
        
        user.setLoginDate(DateUtils.getNowDate());
        user.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        cbCompanyUserService.updateById(user);
        return ajax;
    }

    /**
     * 用户注册
     * @param user
     * @return
     */
    @ApiOperation("门户注册")
    @PostMapping("/register")
    public AjaxResult register(@RequestBody RegisterBody user) {
        String username = user.getUsername(), password = user.getPassword();
        sysLoginService.validateCaptcha(username,user.getCode(),user.getUuid());
        if (StringUtils.isNull(username)) {
            Assert.isTrue(false,"用户名不能为空");
        }else if(StringUtils.isNull(password)){
            Assert.isTrue(false,"密码不能为空");
        }else if (0 < cbCompanyUserService.checkCbCompanyUserByuserName(username)){
            Assert.isTrue(false,"注册用户'" + username + "'失败，账号已存在");
        }
        CbCompanyUser companyUser = new CbCompanyUser();
        companyUser.setUserName(username);
        companyUser.setNickName(username);
        companyUser.setPassword(SecurityUtils.encryptPassword(password));
        companyUser.setLoginDate(DateUtils.getNowDate());
        companyUser.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        cbCompanyUserService.save(companyUser);
        return AjaxResult.success("注册成功");
    }

    /**
     * 找回密码
     * @param user
     * @return
     */
    @PortalAuth
    @ApiOperation("门户找回密码")
    @PostMapping("/findPAS")
    public AjaxResult findPAS(@RequestBody RegisterBody user) {
        String username = user.getUsername(), password = user.getPassword();
        sysLoginService.validateCaptcha(username,user.getCode(),user.getUuid());
        CbCompanyUser companyUser = cbCompanyUserService.selectCbCompanyUserByuserName(username);
        if (StringUtils.isEmpty(username)) {
            Assert.isTrue(false,"用户名不能为空");
        }else if (ObjectUtil.isNull(companyUser)){
            Assert.isTrue(false,"用户不存在！");
        }
        companyUser.setPassword(SecurityUtils.encryptPassword(password));
        cbCompanyUserService.updateById(companyUser);
        return AjaxResult.success("修改成功");
    }


}
